#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Утилиты
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################

import io
import random
import numpy as np
import torch
from tqdm import tqdm
from sklearn.metrics import accuracy_score, confusion_matrix
from collections import Counter
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

# Персональные
from openav.modules.nn.models import rearrange

# ######################################################################################################################
# Функции
# ######################################################################################################################


def fix_seeds(seed):
    random.seed(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False


def train_one_epoch(dataloader, optimizer, criterion, model, device):
    running_loss = 0.0
    processed_size = 0.0

    for i, data in enumerate(tqdm(dataloader)):
        audio, video, labels = data
        optimizer.zero_grad()
        audio = rearrange(audio, "b g n l c -> b g c n l")
        video = rearrange(video, "b g1 g2 h w c -> b g1 g2 c h w")
        pred = model(audio.to(device), video.to(device))
        labels = labels.type(torch.LongTensor)
        loss = criterion(pred, labels.to(device))
        loss.backward()
        optimizer.step()

        processing_size = len(labels)
        processed_size += processing_size

        running_loss += loss.item() * processing_size

    avg_loss = running_loss / processed_size

    return avg_loss


def val_one_epoch(dataloader, criterion, model, device):
    running_loss = 0.0
    processed_size = 0.0
    predictions, targets = list(), list()
    with torch.no_grad():
        for i, data in enumerate(tqdm(dataloader)):
            audio, video, labels = data
            audio = rearrange(audio, "b g n l c -> b g c n l")
            video = rearrange(video, "b g1 g2 h w c -> b g1 g2 c h w")
            pred = model(audio.to(device), video.to(device))
            loss = criterion(pred, labels.to(device))
            processing_size = len(labels)
            processed_size += processing_size
            running_loss += loss.item() * processing_size
            pred = torch.argmax(pred, dim=1).cpu().numpy()
            true = labels.cpu().numpy()
            predictions.extend(pred)
            targets.extend(true)
    avg_vloss = running_loss / processed_size
    acc = accuracy_score(targets, predictions)
    return acc, avg_vloss


def save_conf_matrix(
    y_true,
    y_pred,
    name_labels,
    filename,
    figsize_w=2600,
    figsize_h=2600,
    font_size=14,
    dpi=600,
    pad_inches=0,
    font_scale=1,
):
    y_true_2 = []
    y_pred_2 = []
    for idx, val in enumerate(y_pred):
        if val not in set(Counter(y_pred).keys()) ^ set(Counter(y_true).keys()):
            y_true_2.append(y_true[idx])
            y_pred_2.append(val)

    c_m = confusion_matrix(y_true_2, y_pred_2)

    conf_matrix = pd.DataFrame(c_m, name_labels, name_labels)

    group_counts = ["{0:0.0f}".format(value) for value in conf_matrix.values.flatten()]
    group_percentages = [
        "{0:.2%}".format(value) for value in conf_matrix.div(np.sum(conf_matrix, axis=1), axis=0).values.flatten()
    ]

    labels = [f" {v1} \n {v2} " for v1, v2 in zip(group_counts, group_percentages)]
    labels = np.asarray(labels).reshape(c_m.shape)

    px = 1 / plt.rcParams["figure.dpi"]
    fig, ax = plt.subplots(figsize=(figsize_w * px, figsize_h * px))
    sns.set_theme(font_scale=font_scale)
    sns.heatmap(
        conf_matrix,
        cbar=False,
        annot=labels,
        square=True,
        fmt="",
        annot_kws={"size": font_size * font_scale},
        cmap="Blues",
        ax=ax,
    )

    buf = io.BytesIO()
    fig.savefig(buf, format="png", dpi=dpi, bbox_inches="tight", pad_inches=pad_inches)
    buf.seek(0)

    with open(filename, "wb") as f:
        f.write(buf.read())

    plt.close(fig)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Ядро
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import sys  # Доступ к некоторым переменным и функциям Python
import argparse  # Парсинг аргументов и параметров командной строки
import numpy as np  # Научные вычисления
import pandas as pd  # Обработка и анализ данных
import prettytable  # Отображение таблиц в терминале
import colorama  # Цветной текст терминала
import IPython  # Интерактивная оболочка для языка программирования
import torch  # Машинное обучение от Facebook
import torchaudio  # Работа с аудио от Facebook
import torchvision  # Работа с видео от Facebook
import av  # Работа с FFmpeg
import filetype  # Определение типа файла и типа MIME
import logging  # Логирование
import yaml  # Кодирование и декодирование данные в удобном формате
import pymediainfo  # Получение meta данных из медиафайлов
import librosa
import librosa.display
import matplotlib as mpl
import mediapipe as mp
import cv2
import einops
import tqdm
import sklearn
import seaborn as sns
import flask

from datetime import datetime  # Работа со временем
from prettytable import PrettyTable  # Отображение таблиц в терминале
import pkg_resources  # Работа с ресурсами внутри пакетов

from IPython import get_ipython

# Типы данных
from typing import List, Dict, Union, Any, Optional

# Персональные
import openav  # Библиотека в целом
from openav.modules.core.exceptions import TypeMessagesError
from openav.modules.trml.shell import Shell  # Работа с Shell
from openav.modules.core.settings import Settings  # Глобальный файл настроек

from openav.modules.core.settings import COLOR_INFO, COLOR_SIMPLE, COLOR_ERR, COLOR_TRUE

# ######################################################################################################################
# Константы
# ######################################################################################################################
TYPE_MESSAGES: List[str] = ["info", "correct", "error"]  # Типы возможных сообщений


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class CoreMessages(Settings):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._libs_vers: str = self._("Версии установленных библиотек") + self._em
        self._package: str = self._("Пакет")

        self._trac_file: str = self._("Файл")
        self._trac_line: str = self._("Линия")
        self._trac_method: str = self._("Метод")
        self._trac_type_err: str = self._("Тип ошибки")

        self._undefined_message: str = "... " + self._("неопределенное сообщение") + self._em
        self._wrong_type_messages: str = self._("Тип сообщения должен быть одним из") + " {}" + self._em


# ######################################################################################################################
# Ядро модулей
# ######################################################################################################################
@dataclass
class Core(CoreMessages):
    """Класс-ядро модулей

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._ap: Optional[argparse.ArgumentParser] = None  # Парсер для параметров командной строки
        self._args: Optional[Dict[str, Any]] = None  # Аргументы командной строки

        self._space: int = 4  # Значение для количества пробелов в начале текста

        self._df_pkgs: pd.DataFrame = pd.DataFrame()  # DataFrame c версиями установленных библиотек

        #  Регистратор логирования с указанным именем
        self._logger_core: logging.Logger = logging.getLogger(__class__.__name__)

        # ----------------------- Только для внутреннего использования внутри класса

        self.__max_space: int = 24  # Максимальное значение для количества пробелов в начале текста

        # Список цветов для удаления из LOG файлов
        self.__list_of_chars: List[str] = [
            self.color_green,
            self.color_red,
            self.color_blue,
            self.text_bold,
            self.clear_line,
            self.text_end,
        ]

    # ------------------------------------------------------------------------------------------------------------------
    # Свойства
    # ------------------------------------------------------------------------------------------------------------------

    @property
    def is_notebook(self) -> bool:
        """Получение результата определения запуска библиотеки в Jupyter или аналогах

        Returns:
            bool: **True** если библиотека запущена в Jupyter или аналогах, в обратном случае **False**
        """

        return self.__is_notebook()

    @property
    def df_pkgs(self) -> pd.DataFrame:
        """Получение DataFrame c версиями установленных библиотек

        Returns:
            pd.DataFrame: **DataFrame** c версиями установленных библиотек
        """

        return self._df_pkgs

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (приватные)
    # ------------------------------------------------------------------------------------------------------------------

    @staticmethod
    def __is_notebook() -> bool:
        """Определение запуска библиотеки в Jupyter или аналогах

        .. note::
            private (приватный метод)

        Returns:
            bool: **True** если библиотека запущена в Jupyter или аналогах, в обратном случае **False**
        """

        try:
            # Определение режима запуска библиотеки
            shell = get_ipython().__class__.__name__
        except (NameError, Exception):
            return False  # Запуск в Python
        else:
            if shell == "ZMQInteractiveShell" or shell == "Shell":
                return True
            elif shell == "TerminalInteractiveShell":
                return False
            else:
                return False

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    @staticmethod
    def _traceback() -> Dict[str, Union[str, int]]:
        """Трассировка исключений

        .. note::
            protected (защищенный метод)

        Returns:
            Dict[str, Union[str, int]]: Словарь с описанием исключения
        """

        exc_type, exc_value, exc_traceback = sys.exc_info()  # Получение информации об ошибке

        _trac = {
            "filename": exc_traceback.tb_frame.f_code.co_filename,
            "lineno": exc_traceback.tb_lineno,
            "name": exc_traceback.tb_frame.f_code.co_name,
            "type": exc_type.__name__,
        }

        return _trac

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы (сообщения)
    # ------------------------------------------------------------------------------------------------------------------

    def inv_args(self, class_name: str, build_name: str, out: bool = True) -> None:
        """Сообщение об указании неверных типов аргументов

        Args:
            class_name (str): Имя класса
            build_name (str): Имя метода/функции
            out (bool): Печатать процесс выполнения

        Returns:
            None
        """

        if type(out) is not bool:
            out = True

        trac = self._traceback()  # Трассировка исключений

        try:
            # Проверка аргументов
            if type(class_name) is not str or not class_name or type(build_name) is not str or not build_name:
                raise TypeError
        except TypeError:
            class_name, build_name = __class__.__name__, self.inv_args.__name__

        inv_args = self._invalid_arguments.format(class_name + "." + build_name)

        if self.is_notebook is False:
            if out is True:
                print(
                    "[{}{}{}] {}".format(
                        self.color_red, datetime.now().strftime(self._format_time)[:-3], self.text_end, inv_args
                    )
                )

                indent = ("\r" + " " * self._space + "{}\n") * 4

                def trac_text(ind):
                    return ind.format(
                        f'{self._trac_file}: {trac["filename"]}',
                        f'{self._trac_line}: {trac["lineno"]}',
                        f'{self._trac_method}: {trac["name"]}',
                        f'{self._trac_type_err}: {trac["type"]}',
                    )

                sys.stdout.write(trac_text(indent))
                sys.stdout.flush()

                indent = ("\r" + " " * self._space + "{}") * 4

                self._logger_core.error(inv_args + trac_text(indent))

            # if self.logger_gui is True:
            #     cr = COLOR_SIMPLE

            #     message = "{}[{}{}{}] {}{}{}{}{}{}".format(
            #         f'<p style="display:block; margin:0; color:{cr}"><strong>',
            #         f'</span><span style="color:{COLOR_ERR}">',
            #         datetime.now().strftime(self._format_time),
            #         f'</span><span style="color:{cr}">',
            #         inv_args,
            #         "</strong></p>",
            #         f'<p><span style="color:{cr}; padding-left:24px">{self._trac_file}: <u>{trac["filename"]}</u></span>',
            #         f'<br /><span style="color:{cr}; padding-left:24px">{self._trac_line}: <u>{trac["lineno"]}</u></span>',
            #         f'<br /><span style="color:{cr}; padding-left:24px">{self._trac_method}: <u>{trac["name"]}</u></span>',
            #         f'<br /><span style="color:{cr}; padding-left:24px">{self._trac_type_err}: <u>{trac["type"]}</u></span></p>',
            #     )

            #     self._add_logger_messages_gui(message)

    def message_error(
        self, message: str, space: int = 0, start: bool = False, end: bool = True, out: bool = True
    ) -> None:
        """Сообщение об ошибке

        Args:
            message (str): Сообщение
            space (int): Количество пробелов в начале текста
            start (bool): Начинать сообщение переходом на новую строку
            end (bool): Заканчивать сообщение переходом на новую строку
            out (bool): Отображение

        Returns:
            None
        """

        if type(out) is not bool:
            out = True

        trac = self._traceback()  # Трассировка исключений

        try:
            # Проверка аргументов
            if (
                type(message) is not str
                or not message
                or type(space) is not int
                or not (0 <= space <= self.__max_space)
                or type(start) is not bool
                or type(end) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.message_error.__name__, out=out)
            return None

        if self.is_notebook is False:
            if out is True:
                ns = ""
                ne = "\n"

                if start is True:
                    ns = "\n"

                if end is False:
                    ne = "\r"

                print(
                    ns
                    + " " * space
                    + "[{}{}{}] {}".format(
                        self.color_red, datetime.now().strftime(self._format_time)[:-3], self.text_end, message
                    ),
                    end=ne,
                )

                indent = ("\r" + " " * self._space + "{}\n") * 4

                def trac_text(ind):
                    return ind.format(
                        f'{self._trac_file}: {trac["filename"]}',
                        f'{self._trac_line}: {trac["lineno"]}',
                        f'{self._trac_method}: {trac["name"]}',
                        f'{self._trac_type_err}: {trac["type"]}',
                    )

                sys.stdout.write(trac_text(indent))
                sys.stdout.flush()

                indent = ("\r" + " " * self._space + "{}") * 4

                for character in self.__list_of_chars:
                    message = message.replace(character, "")
                self._logger_core.error(message + trac_text(indent))

    def message_true(
        self, message: str, space: int = 0, start: bool = False, end: bool = True, out: bool = True
    ) -> None:
        """Сообщение с положительной информацией

        Args:
            message (str): Сообщение
            space (int): Количество пробелов в начале текста
            start (bool): Начинать сообщение переходом на новую строку
            end (bool): Заканчивать сообщение переходом на новую строку
            out (bool): Отображение

        Returns:
            None
        """

        if type(out) is not bool:
            out = True

        try:
            # Проверка аргументов
            if (
                type(message) is not str
                or not message
                or type(space) is not int
                or not (0 <= space <= self.__max_space)
                or type(start) is not bool
                or type(end) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.message_true.__name__, out=out)
            return None

        if self.is_notebook is False:
            if out is True:
                ns = ""
                ne = "\n"

                if start is True:
                    ns = "\n"

                if end is False:
                    ne = "\r"

                print(
                    ns
                    + " " * space
                    + "[{}{}{}] {}".format(
                        self.color_green, datetime.now().strftime(self._format_time)[:-3], self.text_end, message
                    ),
                    end=ne,
                )

                for character in self.__list_of_chars:
                    message = message.replace(character, "")
                self._logger_core.info(message)

    def message_info(
        self, message: str, space: int = 0, start: bool = False, end: bool = True, out: bool = True
    ) -> None:
        """Информационное сообщение

        Args:
            message (str): Сообщение
            space (int): Количество пробелов в начале текста
            start (bool): Начинать сообщение переходом на новую строку
            end (bool): Заканчивать сообщение переходом на новую строку
            out (bool): Отображение

        Returns:
            None
        """

        if type(out) is not bool:
            out = True

        try:
            # Проверка аргументов
            if (
                type(message) is not str
                or not message
                or type(space) is not int
                or not (0 <= space <= self.__max_space)
                or type(start) is not bool
                or type(end) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.message_info.__name__, out=out)
            return None

        if self.is_notebook is False:
            if out is True:
                ns = ""
                ne = "\n"

                if start is True:
                    ns = "\n"

                if end is False:
                    ne = "\r"

                print(
                    ns + " " * space + "[{}] {}".format(datetime.now().strftime(self._format_time)[:-3], message),
                    end=ne,
                )

                for character in self.__list_of_chars:
                    message = message.replace(character, "")
                self._logger_core.info(message)

    def message_line(self, message: str, type_message: str = TYPE_MESSAGES[0], out: bool = True) -> str:
        """Информационное сообщение (в виде одной строки)

        Args:
            message (str): Сообщение
            type_message (str): Тип сообщения
            out (bool): Отображение

        Returns:
            str: Информационное сообщение (в виде одной строки)
        """

        if type(out) is not bool:
            out = True

        try:
            # Проверка аргументов
            if type(message) is not str or not message:
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.message_line.__name__, out=out)
            return self._undefined_message
        else:
            try:
                # Проверка типа сообщения
                if type(type_message) is not str or (type_message in TYPE_MESSAGES) is False:
                    raise TypeMessagesError
            except TypeMessagesError:
                self.message_error(
                    self._wrong_type_messages.format(", ".join(x.replace(".", "") for x in TYPE_MESSAGES)), out=out
                )
                self._undefined_message
            else:
                # Тип сообщения
                if type_message == TYPE_MESSAGES[0]:
                    tm = self.color_blue
                elif type_message == TYPE_MESSAGES[1]:
                    tm = self.color_green
                elif type_message == TYPE_MESSAGES[2]:
                    tm = self.color_red
                else:
                    tm = self.text_bold

                if self.is_notebook is False:
                    if out is True:
                        return ("{}" * 3).format(tm, message, self.text_end)

                return self._undefined_message

    def message_metadata_info(self, out: bool = True) -> None:
        """Информация об библиотеке

        Args:
            out (bool): Отображение

        Returns:
            None
        """

        if type(out) is not bool:
            out = True

        if self.is_notebook is False:
            space = " " * self._space

            def generate_name_with_email(list1, list2):
                return "".join(
                    map(str, map(lambda l1, l2: f"{space * 2}{l1} [{l2}]\n", list1.split(", "), list2.split(", ")))
                )

            author = generate_name_with_email(
                openav.__author__ru__ if self.lang == "ru" else openav.__author__en__, openav.__email__
            )

            maintainer = generate_name_with_email(
                openav.__maintainer__ru__ if self.lang == "ru" else openav.__maintainer__en__,
                openav.__maintainer_email__,
            )

            if out is True:

                def metadata():
                    return ("{}" * 5).format(
                        f"{self._metadata[0]}:\n",
                        f"{space}{self._metadata[1]}:\n{author}",
                        f"{space}{self._metadata[2]}:\n{maintainer}",
                        f"{space}{self._metadata[3]}: {openav.__release__}\n",
                        f"{space}{self._metadata[4]}: {openav.__license__}",
                    )

                print("{}".format(f"[{datetime.now().strftime(self._format_time)[:-3]}] {metadata()}"))
                self._logger_core.info(metadata())

    def message_progressbar(self, message: str = "", space: int = 0, close: bool = False, out: bool = True) -> str:
        """Информационный индикатор выполнения

        Args:
            message (str): Сообщение
            space (int): Количество пробелов в начале текста
            close (bool): Закрыть информационный индикатор
            out (bool): Отображение

        Returns:
            None
        """

        if type(out) is not bool:
            out = True
        if type(close) is not bool:
            close = False

        if close is True:
            message = "Закрыть"

        try:
            # Проверка аргументов
            if (
                type(message) is not str
                or not message
                or type(space) is not int
                or not (0 <= space <= self.__max_space)
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.message_progressbar.__name__, out=out)
            return None

        if self.is_notebook is False:
            if out is True:
                if close is False:
                    message_log = message
                    for character in self.__list_of_chars:
                        message_log = message_log.replace(character, "")
                    self._logger_core.info(message_log)

                message = (
                    "\r"
                    + self.clear_line
                    + (" " * space)
                    + "[{}] {}".format(datetime.now().strftime(self._format_time)[:-3], message)
                )

                if close is True:
                    message = "\n"

                sys.stdout.write(message)
                sys.stdout.flush()

    # ------------------------------------------------------------------------------------------------------------------
    # Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def libs_vers(self, out: bool = True) -> bool:
        """Получение и отображение версий установленных библиотек

        Args:
            out (bool): Отображение

        Returns:
            bool: **True** если версии установленных библиотек отображены, в обратном случае **False**
        """

        # Сброс
        self._df_pkgs = pd.DataFrame()  # Пустой DataFrame

        try:
            # Проверка аргументов
            if type(out) is not bool:
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.libs_vers.__name__, out=out)
            return False
        else:
            pkgs = {
                "Package": [
                    "PyTorch",
                    "TorchAudio",
                    "TorchVision",
                    "NumPy",
                    "Pandas",
                    "Matplotlib",
                    "Seaborn",
                    "PyAV",
                    "FileType",
                    "IPython",
                    "Colorama",
                    "Prettytable",
                    "PyYAML",
                    "PyMediaInfo",
                    "Librosa",
                    "MediaPipe",
                    "OpenCV",
                    "Flask",
                    "Einops",
                    "Tqdm",
                    "Scikit-learn",
                    "lion-pytorch",
                    "Streamlit",
                    "Vosk",
                ],
                "Version": [
                    i.__version__
                    for i in [
                        torch,
                        torchaudio,
                        torchvision,
                        np,
                        pd,
                        mpl,
                        sns,
                        av,
                        filetype,
                        IPython,
                        colorama,
                        prettytable,
                        yaml,
                        pymediainfo,
                        librosa,
                        mp,
                        cv2,
                        flask,
                        einops,
                        tqdm,
                        sklearn,
                    ]
                ],
            }
            pkgs["Version"].append(pkg_resources.get_distribution("lion_pytorch").version)
            pkgs["Version"].append(pkg_resources.get_distribution("streamlit").version)
            pkgs["Version"].append(pkg_resources.get_distribution("vosk").version)

            self._df_pkgs = pd.DataFrame(data=pkgs)  # Версии используемых библиотек
            self._df_pkgs.index += 1

            if self.is_notebook is False:
                # Вывод сообщения
                if out is True:
                    table_terminal = PrettyTable()
                    table_terminal.add_column(self._package, self._df_pkgs["Package"].values)
                    table_terminal.add_column(self._metadata[3], self._df_pkgs["Version"].values)
                    table_terminal.align = "l"

                    table_terminal = "\r\n" + table_terminal.__str__()

                    self.message_info(self._libs_vers + table_terminal, space=0, out=out)

    def build_args(self, description: str, conv_to_dict: bool = True, out: bool = True) -> Dict[str, Any]:
        """Построение аргументов командной строки

        Args:
            description (str): Описание парсера командной строки
            conv_to_dict (bool): Преобразование списка аргументов командной строки в словарь
            out (bool): Печатать процесс выполнения

        Returns:
            Dict[str, Any]: Словарь со списком аргументов командной стройки
        """

        try:
            # Проверка аргументов
            if (
                type(description) is not str
                or not description
                or type(conv_to_dict) is not bool
                or type(out) is not bool
            ):
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.build_args.__name__, out=out)
            return {}
        else:
            # Парсер для параметров командной строки
            self._ap = argparse.ArgumentParser(description=description)

            if conv_to_dict is True:
                return vars(self._ap.parse_args())  # Преобразование списка аргументов командной строки в словарь

    def clear_shell(self, cls: bool = True, out: bool = True) -> bool:
        """Очистка консоли

        Args:
            cls (bool): Очистка консоли
            out (bool): Печатать процесс выполнения

        Returns:
            bool: **True** если консоль очищена, в обратном случае **False**
        """

        try:
            # Проверка аргументов
            if type(cls) is not bool or type(out) is not bool:
                raise TypeError
        except TypeError:
            self.inv_args(__class__.__name__, self.clear_shell.__name__, out=out)
            return False
        else:
            if cls is True:
                Shell.clear()  # Очистка консоли
            return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Архитектуры нейросетевых моделей
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################

import math
import torch
import torch.nn as nn
import torch.nn.functional as F
from einops import rearrange

# ######################################################################################################################
# Модели
# ######################################################################################################################


class Bottleneck(nn.Module):
    expansion = 4

    def __init__(self, in_channels, out_channels, i_downsample=None, stride=1):
        super(Bottleneck, self).__init__()

        self.i_downsample = i_downsample

        if self.i_downsample is not None:
            self.conv1 = Conv2dSame(in_channels, out_channels, 3, stride=stride, groups=1, bias=True)
        else:
            self.conv1 = nn.Conv2d(in_channels, out_channels, kernel_size=3, stride=stride, padding="same", bias=True)
        self.batch_norm1 = nn.BatchNorm2d(out_channels, eps=0.001, momentum=0.99)

        self.conv2 = nn.Conv2d(out_channels, out_channels, kernel_size=3, padding="same", bias=True)
        self.batch_norm2 = nn.BatchNorm2d(out_channels, eps=0.001, momentum=0.99)

        self.relu = nn.ReLU()

    def forward(self, x):
        identity = x.clone()
        x = self.relu(self.batch_norm1(self.conv1(x)))

        x = self.relu(self.batch_norm2(self.conv2(x)))

        if self.i_downsample is not None:
            identity = self.i_downsample(identity)
        x += identity
        x = self.relu(x)

        return x


class Conv2dSame(torch.nn.Conv2d):
    def calc_same_pad(self, i: int, k: int, s: int, d: int) -> int:
        return max((math.ceil(i / s) - 1) * s + (k - 1) * d + 1 - i, 0)

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        ih, iw = x.size()[-2:]

        pad_h = self.calc_same_pad(i=ih, k=self.kernel_size[0], s=self.stride[0], d=self.dilation[0])
        pad_w = self.calc_same_pad(i=iw, k=self.kernel_size[1], s=self.stride[1], d=self.dilation[1])

        if pad_h > 0 or pad_w > 0:
            x = F.pad(x, [pad_w // 2, pad_w - pad_w // 2, pad_h // 2, pad_h - pad_h // 2])
        return F.conv2d(
            x,
            self.weight,
            self.bias,
            self.stride,
            self.padding,
            self.dilation,
            self.groups,
        )


class ResNet(nn.Module):
    def __init__(self, ResBlock, layer_list, num_channels=3):
        super(ResNet, self).__init__()
        self.in_channels = 64

        self.conv_layer_s2_same = Conv2dSame(num_channels, 64, 7, stride=2, groups=1, bias=False)
        self.batch_norm1 = nn.BatchNorm2d(64, eps=0.001, momentum=0.99)
        self.relu = nn.ReLU()
        self.max_pool = nn.MaxPool2d(kernel_size=2, stride=2)

        self.layer1 = self._make_layer(ResBlock, layer_list[0], planes=64, stride=1)
        self.layer2 = self._make_layer(ResBlock, layer_list[1], planes=128, stride=2)
        self.layer3 = self._make_layer(ResBlock, layer_list[2], planes=256, stride=2)
        self.layer4 = self._make_layer(ResBlock, layer_list[3], planes=512, stride=2)

        self.avgpool = nn.AdaptiveAvgPool2d((1, 1))

    def extract_features(self, x):
        x = self.relu(self.batch_norm1(self.conv_layer_s2_same(x)))
        x = self.max_pool(x)
        x = self.layer1(x)
        x = self.layer2(x)
        x = self.layer3(x)
        x = self.layer4(x)
        x = self.avgpool(x)
        x = x.reshape(x.shape[0], -1)
        return x

    def forward(self, x):
        x = self.extract_features(x)
        return x

    def _make_layer(self, ResBlock, blocks, planes, stride=1):
        ii_downsample = None
        layers = []

        if stride != 1 or self.in_channels != planes:
            ii_downsample = nn.Sequential(
                Conv2dSame(self.in_channels, planes, 1, stride=stride, groups=1, bias=True),
                nn.BatchNorm2d(planes, eps=0.001, momentum=0.99),
            )

        layers.append(ResBlock(self.in_channels, planes, i_downsample=ii_downsample, stride=stride))
        self.in_channels = planes

        for i in range(blocks - 1):
            layers.append(ResBlock(self.in_channels, planes))

        return nn.Sequential(*layers)


def ResNet18(channels=3):
    return ResNet(Bottleneck, [2, 2, 2, 2], channels)


class CNNLSTMPyTorch(nn.Module):
    def __init__(self, h_u=512, channels=3, p_drop=0.2):
        super(CNNLSTMPyTorch, self).__init__()
        self.resnet = ResNet18(channels=channels)
        self.lstm1 = nn.LSTM(input_size=h_u, hidden_size=h_u, batch_first=True, bidirectional=False)
        self.drop1 = nn.Dropout(p=p_drop)
        self.lstm2 = nn.LSTM(input_size=h_u, hidden_size=h_u, batch_first=True, bidirectional=False)
        self.drop2 = nn.Dropout(p=p_drop)

    def forward(self, x):
        x_n = rearrange(x, "b g c h w-> (b g) c h w")
        x_n = self.resnet(x_n)
        x_n = rearrange(x_n, "(b g) f-> b g f", b=x.shape[0], g=x.shape[1])
        x_n, _ = self.lstm1(x_n)
        x_n = self.drop1(x_n)
        x_n, _ = self.lstm2(x_n)
        x_n = x_n[:, -1, :]
        x_n = self.drop2(x_n)
        return x_n


class Attention(nn.Module):
    def __init__(self, input_dim):
        super(Attention, self).__init__()
        self.input_dim = input_dim
        self.query = nn.Linear(input_dim, input_dim)
        self.key = nn.Linear(input_dim, input_dim)
        self.value = nn.Linear(input_dim, input_dim)
        self.softmax = nn.Softmax(dim=2)

    def forward(self, x1, x2, x3):
        queries = self.query(x1)
        keys = self.key(x2)
        values = self.value(x3)

        scores = torch.bmm(queries, keys.transpose(1, 2)) / (self.input_dim**0.5)
        attention = self.softmax(scores)
        weighted = torch.bmm(attention, values)
        return weighted


class InitialEncoder(nn.Module):
    def __init__(self, input_dim=512):
        super(InitialEncoder, self).__init__()
        self.att = Attention(input_dim)
        self.lambda_ = nn.Parameter(torch.tensor(1, dtype=torch.float32))
        self.ln1 = nn.LayerNorm(input_dim)
        self.ln2 = nn.LayerNorm(input_dim)
        self.fc1 = nn.Linear(input_dim, input_dim)
        self.fc2 = nn.Linear(input_dim, input_dim)

    def forward(self, x1):
        x_w = self.att(x1, x1, x1)
        x_w = x1 * self.lambda_ + x_w * (1 - self.lambda_)
        x_w = self.ln1(x_w)
        x_w_fc1 = self.fc1(x_w)
        x_w = self.ln2(x_w + x_w_fc1)
        x_w = self.fc2(x_w)
        return x_w


class SecondEncoder(nn.Module):
    def __init__(self, input_dim=512):
        super(SecondEncoder, self).__init__()
        self.att1 = Attention(input_dim)
        self.att2 = Attention(input_dim)
        self.lambda_ = nn.Parameter(torch.tensor(1, dtype=torch.float32))
        self.ln1 = nn.LayerNorm(input_dim)
        self.ln2 = nn.LayerNorm(input_dim)
        self.fc1 = nn.Linear(input_dim, input_dim)
        self.fc2 = nn.Linear(input_dim, input_dim)

    def forward(self, x1, x2):
        x_w1 = self.att1(x1, x1, x1)
        x_w2 = self.att2(x1, x2, x2)
        x_w12 = x_w1 * self.lambda_ + x_w2 * (1 - self.lambda_)
        x_w12 = self.ln1(x1 + x_w12)
        x_w12_fc1 = self.fc1(x_w12)
        x_w12 = self.ln2(x_w12 + x_w12_fc1)
        x_w12 = self.fc2(x_w12)
        return x_w12


class Decoder(nn.Module):
    def __init__(self, input_dim=512, h_u=512, n_class=500):
        super(Decoder, self).__init__()
        self.avgpool = nn.AdaptiveAvgPool1d(1)
        self.fc1 = nn.Linear(input_dim, h_u)
        self.fc2 = nn.Linear(h_u, n_class)
        self.softmax = nn.Softmax(dim=1)

    def forward(self, x):
        x = x.transpose(1, 2)
        x = self.avgpool(x).transpose(1, 2)
        x = x.reshape(x.shape[0], -1)
        x = nn.functional.relu(self.fc1(x))
        x = self.fc2(x)
        x = self.softmax(x)
        return x


class Transformer(nn.Module):
    def __init__(self, input_dim=512, h_u=512, n_class=500, encoder_decoder=5):
        super(Transformer, self).__init__()

        self.encoders = nn.ModuleList()
        self.decoders = nn.ModuleList()

        # Создание энкодеров
        self.encoders.append(InitialEncoder(input_dim=input_dim))
        for _ in range(encoder_decoder - 1):
            self.encoders.append(SecondEncoder(input_dim=input_dim))

        # Создание декодеров
        for _ in range(encoder_decoder):
            self.decoders.append(Decoder(input_dim=input_dim, h_u=h_u, n_class=n_class))

    def forward(self, x):
        x_w = [x]

        x_w.append(self.encoders[0](x))

        for encoder in self.encoders[1:]:
            x_w.append(encoder(x, x_w[-1]))

        y_all = []
        for i, decoder in enumerate(self.decoders):
            y_all.append(decoder(x_w[i]))

        y_all = torch.stack(y_all)
        y_mean = torch.mean(y_all, dim=0)

        return y_mean


class AVModel(nn.Module):
    def __init__(self, shape_audio, shape_video, input_dim=512, h_u=512, h_f=64, n_class=500, encoder_decoder=5):
        super(AVModel, self).__init__()
        self.feature_audio = ResNet18(channels=shape_audio[2])
        self.fc_audio = nn.Linear(shape_audio[1], h_f)
        self.feature_video = CNNLSTMPyTorch()
        self.fc_video = nn.Linear(shape_video[1], h_f)
        self.fusion = Transformer(input_dim=input_dim, h_u=h_u, n_class=n_class, encoder_decoder=encoder_decoder)

    def forward(self, audio, video):
        audio_n = rearrange(audio, "b g c n l-> (b g) c n l")
        audio_n = rearrange(self.feature_audio(audio_n), "(b g) f -> b g f", b=audio.shape[0], g=audio.shape[1])
        audio_n = self.fc_audio(audio_n.transpose(1, 2)).transpose(1, 2)
        video_n = rearrange(video, "b g1 g2 c h w-> (b g1) g2 c h w")
        video_n = rearrange(self.feature_video(video_n), "(b g) f -> b g f", b=video.shape[0], g=video.shape[1])
        video_n = self.fc_video(video_n.transpose(1, 2)).transpose(1, 2)
        av = torch.cat([audio_n, video_n], dim=1)
        out = self.fusion(av)
        return out



from flask import Flask, request, render_template, jsonify, send_file
import os
import csv
from werkzeug.middleware.proxy_fix import ProxyFix

app = Flask(__name__)
app.wsgi_app = ProxyFix(app.wsgi_app, x_for=1, x_proto=1, x_host=1, x_prefix=1)
global video_path, output_dir
global processing_finished

processing_finished = False
video_path = "temp_dir/video.mp4"
output_dir = "cuted"


def read_questions_from_csv():
    questions = []
    with open("questions.csv", mode="r", newline="", encoding="utf-8") as file:
        csv_reader = csv.DictReader(file)
        for row in csv_reader:
            question_number = int(row["QuestionNumber"])
            question_text = row["QuestionText"]
            disable_time = row["Disable_time"]
            questions.append(
                {"QuestionNumber": question_number, "QuestionText": question_text, "Disable_time": disable_time}
            )
    return questions


@app.route("/get_questions", methods=["GET"])
def get_questions():
    """Получение списка вопросов

    GET /get_questions

    Возвращает список вопросов в json формате
    """
    questions = read_questions_from_csv()
    return jsonify(questions=questions)


# Configure the upload folder for recorded videos
app.config["UPLOAD_FOLDER"] = "static/recorded-video"

if not os.path.exists(app.config["UPLOAD_FOLDER"]):
    os.makedirs(app.config["UPLOAD_FOLDER"])


@app.route("/")
def index():
    """Отображение основной страницы записи

    GET /

    Отображает статику страницы записи. Является точкой входа в приложение
    """
    global processing_finished
    processing_finished = False
    return render_template("index_home.html")


@app.route("/store_timing_data", methods=["POST"])
def store_timing_data():
    """Сохранение файла временных отметок записи

    POST /store_timing_data

    Сохраняет файл временных отметок записи, который предается в виде json в теле запроса
    """
    if request.method == "POST":
        data = request.json
        if os.path.exists("timing_data.txt"):
            os.remove("timing_data.txt")
        with open("timing_data.txt", "w", encoding="utf-8") as file:
            for item in data:
                file.write(f"Question: {item['question']}, Timestamp: {item['timestamp']}\n")

            file.close()
        return "Data stored successfully"


@app.route("/upload", methods=["POST"])
def upload():
    """Сохранение файла записи

    POST /upload

    Сохраняет файл записи, который предается в виде файла видео в теле запроса
    """
    global processing_finished
    if "video" in request.files:
        video_file = request.files["video"]
        if video_file.filename != "":
            video_path = os.path.join(app.config["UPLOAD_FOLDER"], video_file.filename) + ".webm"
            print(video_path)
            # video_file.save(video_path)
            if not os.path.exists("temp_dir"):
                os.makedirs("temp_dir")
            video_file.save("temp_dir/video.webm")
            # convert_webm_to_mp4('temp_dir/video.webm', 'temp_dir/video.mp4')
            print("Done")
            processing_finished = True

            return "DONE"

    return "No video data received."


@app.route("/download_processed_video")
def download_processed_video():
    """Выгрузка файла записи

    GET /download_processed_video

    Скачивает файл с записью в формате webm
    """
    processed_video_path = "temp_dir/video.webm"
    return send_file(processed_video_path, as_attachment=True)


@app.route("/download_timing_data")
def download_timing_data():
    """Выгрузка файла временных отметок записи

    GET /download_processed_video

    Скачивает файл временных отметок записи в формате txt
    """
    timing_data_path = "timing_data.txt"  # Path to the file
    # Set cache-control headers to prevent caching
    response = send_file(timing_data_path, as_attachment=True, mimetype="text/plain")
    response.headers["Cache-Control"] = "no-store, no-cache, must-revalidate, max-age=0"
    return response


if __name__ == "__main__":
    app.run()



__all__ = (
    "__title__",
    "__summary__",
    "__uri__",
    "__version__",
    "__author__ru__",
    "__author__en__",
    "__email__",
    "__license__",
    "__copyright__",
    "__maintainer__ru__",
    "__maintainer__en__",
    "__maintainer_email__",
)

__title__ = "OpenAV"
__summary__ = "OpenAV"
__uri__ = "https://github.com/DmitryRyumin/openav"

__version__ = "1.0"
__release__ = __version__ + ".0-a20"

__author__ru__ = "Рюмин Дмитрий, Иванько Денис, Шилов Николай, Маркитантов Максим, Карпов Алексей"
__author__en__ = "Dmitry Ryumin, Denis Ivanko, Nikolay Shilov, Maxim Markitantov, Alexey Karpov"
__email__ = (
    "dl_03.03.1991@mail.ru, denis.ivanko11@gmail.com, nick@iias.spb.su, m.markitantov@yandex.ru, " "karpov@iias.spb.su"
)

__maintainer__ru__ = "Рюмин Дмитрий"
__maintainer__en__ = "Dmitry Ryumin"
__maintainer_email__ = "dl_03.03.1991@mail.ru"

__license__ = "MIT"
__copyright__ = "2022-25. Dmitry Ryumin, Denis Ivanko, Nikolay Shilov, Maxim Markitantov, Alexey Karpov"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Детектирование речевой активности и распознавание речи в аудиовизуальном сигнале
"""

import os
import sys

PATH_TO_SOURCE = os.path.abspath(os.path.dirname(__file__))
PATH_TO_ROOT = os.path.join(PATH_TO_SOURCE, "..", "..")

sys.path.insert(0, os.path.abspath(PATH_TO_ROOT))

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
# Подавление Warning
import warnings

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

import logging  # Логирование Функции создающие итераторы для эффективного цикла

# Типы данных
from typing import Dict, Union, Any
from types import ModuleType

# Персональные
import openav  # Библиотека в целом
from openav.modules.trml.shell import Shell  # Работа с Shell
from openav.modules.lab.build import Run  # Сборка библиотеки
from openav import rsrs  # Ресурсы библиотеки

from openav.modules.core.logging import ARG_PATH_TO_LOGS
from openav.modules.lab.audio import (
    TYPES_ENCODE,
    PRESETS_CRF_ENCODE,
    SR_INPUT_TYPES,
    SAMPLING_RATE_VAD,
    VOSK_SUPPORTED_LANGUAGES,
    VOSK_SUPPORTED_DICTS,
)


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class MessagesVoskSR(Run):
    """Класс для сообщений"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._description: str = self._(
            "Детектирование речевой активности и распознавание речи в аудиовизуальном сигнале"
        )
        self._description_time: str = "{}" * 2 + self._description + self._em + "{}"

        self._check_config_file_valid = self._("Проверка данных на валидность") + self._em


# ######################################################################################################################
# Выполняем только в том случае, если файл запущен сам по себе
# ######################################################################################################################
@dataclass
class RunVoskSR(MessagesVoskSR):
    """Класс для детектирования речевой активности и распознавание речи в аудиовизуальном сигнале"""

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._all_layer_in_yaml = 20  # Общее количество настроек в конфигурационном файле

        #  Регистратор логирования с указанным именем
        self._logger_run_vosk_sr: logging.Logger = logging.getLogger(__class__.__name__)

    # ------------------------------------------------------------------------------------------------------------------
    # Внутренние методы (защищенные)
    # ------------------------------------------------------------------------------------------------------------------

    def _build_args(self, description: str, conv_to_dict: bool = True, out=True) -> Dict[str, Any]:
        """
        Args:
            description (str): Описание парсера командной строки
            conv_to_dict (bool): Преобразование списка аргументов командной строки в словарь
            out (bool): Печатать процесс выполнения

        Returns:
            Dict[str, Any]: Словарь со списком аргументов командной стройки
        """

        # Выполнение функции из суперкласса
        super().build_args(description=description, conv_to_dict=False, out=out)

        if self._ap is None:
            return {}

        # Добавление аргументов в парсер командной строки
        self._ap.add_argument(
            "--config", required=True, metavar=self._("ФАЙЛ"), help=self._("Путь к конфигурационному файлу")
        )

        self._ap.add_argument(
            ARG_PATH_TO_LOGS,
            required=False,
            metavar=self._("ФАЙЛ"),
            help=self._("Путь к директории для сохранения LOG файлов"),
        )

        self._ap.add_argument(
            "--automatic_update",
            action="store_true",
            help=self._(
                "Автоматическая проверка конфигурационного файла в момент работы программы (работает при заданном"
            )
            + " --config",
        )
        self._ap.add_argument(
            "--no_clear_shell", action="store_false", help=self._("Не очищать консоль перед выполнением")
        )

        # Преобразование списка аргументов командной строки в словарь
        if conv_to_dict is True:
            args, _ = self._ap.parse_known_args()
            return vars(args)  # Преобразование списка аргументов командной строки в словарь

    def _valid_yaml_config(self, config: Dict[str, Union[str, bool, int, float]], out: bool = True) -> bool:
        """Проверка настроек JSON на валидность

        Args:
            config (Dict[str, Union[str, bool, int, float]]): Словарь из JSON файла
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если конфигурационный файл валиден, в обратном случае **False**
        """

        # Проверка аргументов
        if type(config) is not dict or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self._valid_yaml_config.__name__, out=out)
                return False

        # Конфигурационный файл пуст
        if not config:
            try:
                raise TypeError
            except TypeError:
                self.message_error(self._config_empty, space=self._space, out=out)
                return False

        # Вывод сообщения
        self.message_info(self._check_config_file_valid, space=self._space, out=out)

        curr_valid_layer = 0  # Валидное количество разделов

        # Проход по всем разделам конфигурационного файла
        for key, val in config.items():
            # 1. Скрытие метаданных
            # 2. Скрытие версий установленных библиотек
            # 3. Принудительная загрузка модели из сети
            # 4. Очистка директории для сохранения фрагментов аудиовизуального сигнала
            if key == "hide_metadata" or key == "hide_libs_vers" or key == "force_reload" or key == "clear_dirvosk_sr":
                # Проверка значения
                if type(val) is not bool:
                    continue

                curr_valid_layer += 1

            # Тип кодирования
            if key == "type_encode":
                # Проверка значения
                if type(val) is not str or (val in TYPES_ENCODE) is False:
                    continue

                curr_valid_layer += 1

            # Качество кодирования
            if key == "crf_value":
                # Проверка значения
                if type(val) is not int or not (0 <= val <= 51):
                    continue

                curr_valid_layer += 1

            # Скорость кодирования и сжатия
            if key == "presets_crf_encode":
                # Проверка значения
                if type(val) is not str or (val in PRESETS_CRF_ENCODE) is False:
                    continue

                curr_valid_layer += 1

            # Тип файлов для распознавания речи
            if key == "sr_input_type":
                # Проверка значения
                if type(val) is not str or (val in SR_INPUT_TYPES) is False:
                    continue

                curr_valid_layer += 1

            # 1. Путь к директории для сохранения моделей
            # 2. Путь к директории набора данных
            # 3. Путь к директории набора данных состоящего из фрагментов аудиовизуального сигнала
            # 4. Имя директории для разархивирования
            if (
                key == "path_to_save_models"
                or key == "path_to_dataset"
                or key == "path_to_dataset_vosk_sr"
                or key == "folder_name_unzip"
            ):
                # Проверка значения
                if type(val) is not str or not val:
                    continue

                curr_valid_layer += 1

            # Глубина иерархии для получения данных
            if key == "depth":
                # Проверка значения
                if type(val) is not int or not (1 <= val <= 10):
                    continue

                curr_valid_layer += 1

            # Частота дискретизации
            if key == "sampling_rate":
                # Проверка значения
                if type(val) is not int or (val in SAMPLING_RATE_VAD) is False:
                    continue

                curr_valid_layer += 1

            # Названия директорий для видео и аудио
            if key == "dir_va_names":
                all_layer_2 = 2  # Общее количество подразделов в текущем разделе
                curr_valid_layer_2 = 0  # Валидное количество подразделов в текущем разделе

                # Проверка значения
                if type(val) is not dict or len(val) == 0:
                    continue

                # Проход по всем подразделам текущего раздела
                for k, v in val.items():
                    # Проверка значения
                    if type(v) is not str or not v:
                        continue

                    # 1. Директории с разделенными видеофрагментами
                    # 2. Директории с разделенными аудиофрагментами
                    if k == "video" or k == "audio":
                        curr_valid_layer_2 += 1

                if all_layer_2 == curr_valid_layer_2:
                    curr_valid_layer += 1

            # Расширения искомых файлов
            if key == "ext_search_files":
                curr_valid_layer_2 = 0  # Валидное количество подразделов в текущем разделе

                # Проверка значения
                if type(val) is not list or len(val) == 0:
                    continue

                # Проход по всем подразделам текущего раздела
                for v in val:
                    # Проверка значения
                    if type(v) is not str or not v:
                        continue

                    curr_valid_layer_2 += 1

                if curr_valid_layer_2 > 0:
                    curr_valid_layer += 1

            # 1. Внутренний левый отступ для итоговых речевых фрагментов
            # 2. Внутренний правый отступ для итоговых речевых фрагментов
            if key == "speech_left_pad_ms" or key == "speech_right_pad_ms":
                # Проверка значения
                if type(val) is not int or not 0 <= val:
                    continue

                curr_valid_layer += 1

            # Языковая модель
            if key == "lang_model":
                # Проверка значения
                if type(val) is not str or (val in VOSK_SUPPORTED_LANGUAGES) is False:
                    continue

                curr_valid_layer += 1

            # Размер словаря языковой модели
            if key == "dict_size":
                # Проверка значения
                if type(val) is not str or (val in VOSK_SUPPORTED_DICTS) is False:
                    continue

                curr_valid_layer += 1

        # Сравнение общего количества ожидаемых настроек и валидных настроек в конфигурационном файле
        if self._all_layer_in_yaml != curr_valid_layer:
            try:
                raise TypeError
            except TypeError:
                self.message_error(self._invalid_file, space=self._space, out=out)
                return False

        return True  # Результат

    def _load_config_yaml(self, resources: ModuleType = rsrs, config="vosk_sr.yaml", out: bool = True) -> bool:
        """Загрузка и проверка конфигурационного файла

        Args:
            resources (ModuleType): Модуль с ресурсами
            config (str): Конфигурационный файл
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если конфигурационный файл загружен и валиден, в обратном случае **False**
        """

        # Проверка аргументов
        if not isinstance(resources, ModuleType) or type(config) is not str or not config or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self._load_config_yaml.__name__, out=out)
                return False

        # Конфигурационный файл передан
        if self._args["config"] is not None:
            config_yaml = self.load_yaml(self._args["config"], False, out)  # Загрузка YAML файла
        else:
            config_yaml = self.load_yaml_resources(resources, config, out)  # Загрузка YAML файла из ресурсов модуля

        # Конфигурационный файл не загружен
        if not config_yaml:
            return False

        # Проверка конфигурационного файла на валидность
        res_valid_yaml_config = self._valid_yaml_config(config_yaml, out)

        # Конфигурационный файл не валидный
        if res_valid_yaml_config is False:
            return False

        # Проход по всем разделам конфигурационного файла
        for k, v in config_yaml.items():
            self._args[k] = v  # Добавление значения из конфигурационного файла в словарь аргументов командной строки

        return True

    # ------------------------------------------------------------------------------------------------------------------
    #  Внешние методы
    # ------------------------------------------------------------------------------------------------------------------

    def run(self, metadata: ModuleType = openav, resources: ModuleType = rsrs, out: bool = True) -> bool:
        """Запуск детектирования речевой активности и распознавания речи в аудиовизуальном сигнале

        Args:
            metadata (ModuleType): Модуль из которого необходимо извлечь информацию
            resources (ModuleType): Модуль с ресурсами
            out (bool): Печатать процесс выполнения

        Returns:
             bool: **True** если детектирование речевой активности в аудиовизуальном сигнале произведено успешно,
                   в обратном случае **False**
        """

        # Проверка аргументов
        if not isinstance(metadata, ModuleType) or not isinstance(resources, ModuleType) or type(out) is not bool:
            try:
                raise TypeError
            except TypeError:
                self.inv_args(__class__.__name__, self.run.__name__, out=out)
                return False

        self._args = self._build_args(self._description)  # Построение аргументов командной строки
        if len(self._args) == 0:
            return False

        # Очистка консоли перед выполнением
        if self.clear_shell(cls=self._args["no_clear_shell"], out=True) is False:
            return False

        # Вывод сообщения
        if out is True:
            # Приветствие
            Shell.add_line()  # Добавление линии во весь экран
            print(self._description_time.format(self.text_bold, self.color_blue, self.text_end))
            self._logger_run_vosk_sr.info(self._description)
            Shell.add_line()  # Добавление линии во весь экран

        # Загрузка и проверка конфигурационного файла
        if self._load_config_yaml(resources, out=out) is False:
            return False

        # Вывод сообщения
        if out is True:
            Shell.add_line()  # Добавление линии во весь экран

        # Информация об библиотеке
        if self._args["hide_metadata"] is False and out is True:
            self.message_metadata_info(out=out)
            Shell.add_line()  # Добавление линии во весь экран

        # Версии установленных библиотек
        if self._args["hide_libs_vers"] is False and out is True:
            self.libs_vers(out=out)
            Shell.add_line()  # Добавление линии во весь экран

        self.path_to_save_models = self._args["path_to_save_models"]  # Путь к директории для сохранения моделей
        self.path_to_dataset = self._args["path_to_dataset"]  # Путь к директории набора данных
        # Путь к директории набора данных состоящего из фрагментов аудиовизуального сигнала
        self.path_to_dataset_vad = self._args["path_to_dataset_vosk_sr"]
        self.path_to_dataset_vosk_sr = self._args["path_to_dataset_vosk_sr"]
        self.dir_va_names = self._args["dir_va_names"]  # Названия директорий для видео и аудио
        self.ext_search_files = self._args["ext_search_files"]  # Расширения искомых файлов
        self.vosk_language_sr = self._args["lang_model"]  # Языковая модель
        self.vosk_dict_language_sr = self._args["dict_size"]  # Размер словаря языковой модели

        self.vosk_sr(
            depth=self._args["depth"],  # Глубина иерархии для получения данных
            type_encode=self._args["type_encode"],  # Тип кодирования
            crf_value=self._args["crf_value"],  # Качество кодирования
            presets_crf_encode=self._args["presets_crf_encode"],  # Скорость кодирования и сжатия
            new_name=self._args["folder_name_unzip"],  # Имя директории для разархивирования
            # Внутренний левый отступ для итоговых речевых фрагментов
            speech_left_pad_ms=self._args["speech_left_pad_ms"],
            # Внутренний правый отступ для итоговых речевых фрагментов
            speech_right_pad_ms=self._args["speech_right_pad_ms"],
            force_reload=self._args["force_reload"],  # Принудительная загрузка модели из сети
            # Очистка директории для сохранения фрагментов аудиовизуального сигнала
            clear_dirvosk_sr=self._args["clear_dirvosk_sr"],
            out=out,
        )

        return True


def main():
    # Запуск детектирования речевой активности в аудиовизуальном сигнале
    vosk_sr = RunVoskSR(lang="ru", path_to_logs="./openav/logs")
    vosk_sr.run(out=True)


if __name__ == "__main__":
    main()



import sys

from setuptools import setup, find_packages

MIN_PYTHON_VERSION = (3, 10)

if sys.version_info[:2] < MIN_PYTHON_VERSION:
    raise RuntimeError("Python version required = {}.{}".format(MIN_PYTHON_VERSION[0], MIN_PYTHON_VERSION[1]))

import openav

REQUIRED_PACKAGES = [
    "ipython == 8.31.0",
    "colorama == 0.4.6",
    "numpy == 1.26.4",
    "pandas == 2.2.3",
    "prettytable == 3.12.0",
    "torch == 2.2.2",
    "torchaudio == 2.2.2",
    "torchvision == 0.17.2",
    "av == 14.0.1",
    "filetype == 1.2.0",
    "vosk == 0.3.44",
    "requests == 2.32.3",
    "pyyaml == 6.0.2",
    "streamlit == 1.41.1",
    "watchdog == 6.0.0",
    "pymediainfo == 6.1.0",
    "pillow == 11.0.0",
    "imgaug == 0.4.0",
    "ffmpeg == 1.4",
    "librosa == 0.10.2.post1",
    "matplotlib == 3.10.0",
    "mediapipe == 0.10.13",
    "opencv_contrib_python == 4.10.0.84",
    "einops == 0.8.0",
    "lion_pytorch == 0.2.3",
    "scikit-learn == 1.6.0",
    "tqdm == 4.67.1",
    "seaborn == 0.13.2",
    "Flask == 3.1.0",
]

CLASSIFIERS = """\
Development Status :: 3 - Alpha
Natural Language :: Russian
Natural Language :: English
Intended Audience :: Developers
Intended Audience :: Education
Intended Audience :: End Users/Desktop
Intended Audience :: Science/Research
Intended Audience :: Information Technology
Intended Audience :: Science/Research
License :: OSI Approved :: MIT License
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Python :: 3.10
Programming Language :: Python :: 3.11
Programming Language :: Python :: 3.12
Programming Language :: Python :: 3 :: Only
Programming Language :: Python :: Implementation :: CPython
Topic :: Scientific/Engineering
Topic :: Scientific/Engineering :: Mathematics
Topic :: Scientific/Engineering :: Artificial Intelligence
Topic :: Scientific/Engineering :: Image Processing
Topic :: Scientific/Engineering :: Image Recognition
Topic :: Software Development
Topic :: Software Development :: Libraries
Topic :: Software Development :: Libraries :: Python Modules
Topic :: Documentation
Topic :: Documentation :: Sphinx
Topic :: Multimedia :: Sound/Audio
Topic :: Multimedia :: Sound/Audio :: Analysis
Topic :: Multimedia :: Sound/Audio :: Speech
Topic :: Software Development :: Libraries
Topic :: Software Development :: Libraries :: Python Modules
Topic :: Software Development :: Localization
Topic :: Utilities
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: POSIX :: Linux
Framework :: Jupyter
Framework :: Jupyter :: JupyterLab :: 4
Framework :: Sphinx
"""

with open("README.md", "r") as fh:
    long_description = fh.read()

    setup(
        name=openav.__name__,
        packages=find_packages(),
        license=openav.__license__,
        version=openav.__release__,
        author=openav.__author__en__,
        author_email=openav.__email__,
        maintainer=openav.__maintainer__en__,
        maintainer_email=openav.__maintainer_email__,
        url=openav.__uri__,
        description=openav.__summary__,
        long_description=long_description,
        long_description_content_type="text/markdown",
        install_requires=REQUIRED_PACKAGES,
        keywords=[
            "OpenAV",
            "LipReading",
            "SpeechRecognition",
            "SignalProcessing",
            "DataAugmentation",
            "ArtificialNeuralNetworks",
            "DeepMachineLearning",
            "TransferLearning",
            "Statistics",
            "ComputerVision",
            "ArtificialIntelligence",
            "Preprocessing",
        ],
        include_package_data=True,
        classifiers=[_f for _f in CLASSIFIERS.split("\n") if _f],
        python_requires=">=3.9, <4",
        entry_points={
            "console_scripts": [
                "openav_vad = openav.api.vad:main",
                "openav_vosk_sr = openav.api.vosk_sr:main",
                "openav_preprocess_audio = openav.api.preprocess_audio:main",
                "openav_preprocess_video = openav.api.preprocess_video:main",
                "openav_augmentation = openav.api.augmentation:main",
                "openav_download = openav.api.download:main",
                "openav_train_audiovisual = openav.api.train_audiovisual:main",
                "openav_test_audiovisual = openav.api.test_audiovisual:main",
            ],
        },
        project_urls={
            "Bug Reports": "https://github.com/DmitryRyumin/openav/issues",
            "Documentation": "https://openav.readthedocs.io",
            "Source Code": "https://github.com/DmitryRyumin/openav/tree/main/openav",
            "Download": "https://github.com/DmitryRyumin/openav/tags",
        },
    )



# sphinx-intl update -p docs/build/gettext -l en -l ru
# sphinx-build -b gettext docs/source/ docs/build/gettext
# sphinx-build -a -b html -D language=ru ./docs/source ./docs/build/ru


# for front_cnt, _ in enumerate(self.__front):
#     torchaudio.save(
#         self.__part_audio_path[front_cnt],
#         self.__aframes[channel][
#             speech_timestamps[0]['start']:speech_timestamps[0]['end']
#         ].unsqueeze(0),
#         self.__file_metadata['audio_fps']
#     )


# torchaudio.save(
#     '/Users/dl/Desktop/test/' + str(cnt) + '.wav',
#     self.__aframes[channel].unsqueeze(0),
#     self.__file_metadata['audio_fps']
# )


# print(self.__part_video_path, self.__part_audio_path)

# continue
#
# print(speech_timestamps, start_time, end_time)
# print(str(Path(self.__local_path(self.__curr_path)).parent))
# print(self.__aframes[channel][0:10].shape, self.__aframes[channel].unsqueeze(0).shape)
# print(self.__aframes[channel].shape, speech_timestamps[0]['start'], speech_timestamps[0]['end'])
#
# torchaudio.save(
#     '/Users/dl/Desktop/test/' + str(cnt) + '_2.wav',
#     self.__aframes[channel][speech_timestamps[0]['start']:speech_timestamps[0]['end']].unsqueeze(0),
#     self.__file_metadata['audio_fps']
# )


# # Тип файла
# kind = filetype.guess(self.__curr_path)
#
# # Видео
# if kind.mime.startswith('video/') is True: pass
#
#
#
# continue

# import subprocess  # Работа с процессами
# # https://trac.ffmpeg.org/wiki/audio%20types
# # Выполнение в новом процессе
# with subprocess.Popen(
#         ['ffmpeg', '-loglevel', 'quiet', '-i', str(self.__curr_path)] + [] +
#         ['-ar', str(16000), '-ac', str(1), '-f', 's16le', '-'],
#         stdout = subprocess.PIPE) as process:
#     t = None
#
#     while True:
#         data = process.stdout.read(4000)
#         if len(data) == 0: break
#
#         if t is None:
#             t = torch.frombuffer(data, dtype = torch.float32)
#         else:
#             t = torch.cat((t, torch.frombuffer(data, dtype = torch.float32)), 0)
#         # print(t.shape, t[0:10])

# t = t.unsqueeze(0)
# t = t.view(t.shape[0], 1)
# print(type(t), t.shape)
# print(torch.flatten(t).shape)

# torchaudio.save('/Users/dl/Desktop/test/' + str(self.__i) + '.wav', t, 16000)

# Чтение аудиофайла
# self.__wav = read_audio(
#     '/Users/dl/@DmitryRyumin/Databases/LRW/LRW_AUDIO/ABSOLUTELY_00001.wav',
#     sampling_rate = 16000
# )
# # print(type(self.__wav), self.__wav[0:100])
# self.__wav = self.__wav.unsqueeze(0)
# torchaudio.save('/Users/dl/Desktop/test/' + str(self.__i) + '.wav', self.__wav, 16000)

# v, a, h = torchvision.io.read_video(
#     self.__curr_path
# )
#
# print(v.shape)
# print(a.shape)
# print(h)
# kind = filetype.guess(self.__curr_path)
# print(kind)
#
# torchaudio.save('/Users/dl/Desktop/test/' + str(self.__i) + '.wav', a, 16000)



import os
import subprocess


def convert_webm_to_mp4(input_path, output_path):
    line = "ffmpeg -i " + os.getcwd() + "/" + input_path + " " + os.getcwd() + "/" + output_path
    subprocess.run(line)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Сообщения
"""

# ######################################################################################################################
# Импорт необходимых инструментов
# ######################################################################################################################
import warnings  # Подавление Warning

for warn in [UserWarning, FutureWarning]:
    warnings.filterwarnings("ignore", category=warn)

from dataclasses import dataclass  # Класс данных

# Типы данных
from typing import List

# Персональные
from openav.modules.core.language import Language  # Определение языка


# ######################################################################################################################
# Сообщения
# ######################################################################################################################
@dataclass
class Messages(Language):
    """Класс для сообщений

    Args:
        path_to_logs (str): Смотреть :attr:`~openav.modules.core.logging.Logging.path_to_logs`
        lang (str): Смотреть :attr:`~openav.modules.core.language.Language.lang`
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Конструктор
    # ------------------------------------------------------------------------------------------------------------------

    def __post_init__(self):
        super().__post_init__()  # Выполнение конструктора из суперкласса

        self._metadata: List[str] = [
            self._(
                "OpenAV - библиотека распознавания речевых команд на пользовательском словаре с использованием "
                "аудиовизуальных данных диктора"
            ),
            self._("Авторы"),
            self._("Сопровождающие"),
            self._("Версия"),
            self._("Лицензия"),
        ]

        self._format_time: str = "%Y-%m-%d %H:%M:%S.%f"  # Формат времени

        self._em: str = " ..."  # Конец сообщений (End Messages)

        self._invalid_arguments: str = self._("Неверные типы или значения аргументов в") + ' "{}"' + self._em
        self._unknown_err: str = (
            self._("Не обработанную ошибку необходимо проанализировать и выявить причину") + self._em
        )

        self._from_precent: str = self._("из")
        self._curr_progress: str = "{} " + self._from_precent + " {} ({}%)" + self._em + " {}" + self._em
